import os
import signal
import subprocess

from typing import Dict, List

from polyaxon._deploy.operators.docker import DockerOperator
from polyaxon._docker import docker_types
from polyaxon._docker.converter.converters import CONVERTERS
from polyaxon._docker.converter.mixins import MIXIN_MAPPING
from polyaxon._runner.executor import BaseExecutor
from polyaxon._runner.kinds import RunnerKind
from polyaxon._schemas.lifecycle import V1Statuses
from polyaxon.exceptions import PolyaxonAgentError
from polyaxon.logger import logger


class Executor(BaseExecutor):
    MIXIN_MAPPING = MIXIN_MAPPING
    CONVERTERS = CONVERTERS
    RUNNER_KIND = RunnerKind.DOCKER

    def __init__(self):
        super().__init__()
        self._ops = {}

    def _get_manager(self):
        return DockerOperator()

    def _get_op_proc(self, run_uuid: str) -> List[subprocess.Popen]:
        return self._ops.get(run_uuid)

    def create(
        self,
        run_uuid: str,
        run_kind: str,
        resource: List[docker_types.V1Container],
    ) -> Dict:
        logger.info(f"[Executor] Starting operation {run_uuid} {run_kind}.")
        self._ops[run_uuid] = []
        for task in resource:
            logger.info(
                f"[Executor] Starting task container {task.name} {task.image} ."
            )
            proc = self.manager.execute(
                task.get_cmd_args(), env=os.environ, output_only=False
            )
            self._ops[run_uuid].append(proc)
            proc.wait()
            task_status = self._get_task_status(proc)
            message = f"Task container {task.name} {task.image} with id {proc.pid} {task_status}"
            if task_status == V1Statuses.SUCCEEDED:
                logger.info(f"[Executor] {message}")
            else:
                logger.warning(f"[Executor] {message}")
                self._clean_temp_execution_path(run_uuid)
                return {
                    "status": V1Statuses.FAILED,
                    "tasks": self._ops[run_uuid],
                    "message": message,
                }
        self._clean_temp_execution_path(run_uuid)
        return {"status": V1Statuses.SUCCEEDED, "tasks": self._ops[run_uuid]}

    def apply(self, run_uuid: str, run_kind: str, resource: Dict) -> Dict:
        raise PolyaxonAgentError(
            "Docker executor does not support apply method.\n"
            "Run: <kind: {}, uuid: {}>".format(run_kind, run_uuid)
        )

    def stop(self, run_uuid: str, run_kind: str):
        proc = self._get_op_proc(run_uuid)
        if proc.poll() is None:
            # Kill the process tree rooted at the child if it's the leader of its own process
            # group, otherwise just kill the child
            try:
                if proc.pid == os.getpgid(proc.pid):
                    os.killpg(proc.pid, signal.SIGTERM)
                else:
                    proc.terminate()
            except OSError:
                # The child process may have exited before we attempted to terminate it, so we
                # ignore OSErrors raised during child process termination
                _msg = f"Failed to terminate operation {run_kind} {run_uuid} child process PID {proc.pid}"
                logger.debug(_msg)
            proc.wait()

    def clean(self, run_uuid: str, run_kind: str):
        return self.apply(
            run_uuid=run_uuid,
            run_kind=run_kind,
            resource={"metadata": {"finalizers": None}},
        )

    def _get_task_status(self, proc) -> V1Statuses:
        exit_code = proc.poll()
        if exit_code is None:
            return V1Statuses.RUNNING
        if exit_code == 0:
            return V1Statuses.SUCCEEDED
        return V1Statuses.FAILED

    def get(self, run_uuid: str, run_kind: str) -> V1Statuses:
        procs = self._get_op_proc(run_uuid)
        return self._get_task_status(procs[-1])



from typing import List, Optional

from clipped.compact.pydantic import StrictStr
from clipped.config.schema import BaseAllowSchemaModel

from polyaxon._sdk.schemas.v1_organization_member import V1OrganizationMember


class V1ListOrganizationMembersResponse(BaseAllowSchemaModel):
    count: Optional[int]
    results: Optional[List[V1OrganizationMember]]
    previous: Optional[StrictStr]
    next: Optional[StrictStr]



from typing import Optional
from typing_extensions import Annotated

from clipped.compact.pydantic import Field, StrictStr, validate_arguments

from polyaxon._schemas.compatibility import V1Compatibility
from polyaxon._schemas.installation import V1Installation
from polyaxon._schemas.log_handler import V1LogHandler
from polyaxon._sdk.base_api import BaseApi
from polyaxon.exceptions import ApiTypeError


class VersionsV1Api(BaseApi):
    @validate_arguments
    def get_compatibility(
        self,
        uuid: Annotated[StrictStr, Field(..., description="UUid")],
        version: Annotated[StrictStr, Field(..., description="Version")],
        service: Annotated[StrictStr, Field(..., description="Service")],
        **kwargs
    ) -> V1Compatibility:
        """Get compatibility versions

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_compatibility(uuid, version, service, async_req=True)
        >>> result = thread.get()

        :param uuid: UUid (required)
        :type uuid: str
        :param version: Version (required)
        :type version: str
        :param service: Service (required)
        :type service: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: V1Compatibility
        """
        kwargs["_return_http_data_only"] = True
        return self.get_compatibility_with_http_info(uuid, version, service, **kwargs)

    @validate_arguments
    def get_compatibility_with_http_info(
        self,
        uuid: Annotated[StrictStr, Field(..., description="UUid")],
        version: Annotated[StrictStr, Field(..., description="Version")],
        service: Annotated[StrictStr, Field(..., description="Service")],
        **kwargs
    ):
        """Get compatibility versions

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_compatibility_with_http_info(uuid, version, service, async_req=True)
        >>> result = thread.get()

        :param uuid: UUid (required)
        :type uuid: str
        :param version: Version (required)
        :type version: str
        :param service: Service (required)
        :type service: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(V1Compatibility, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = ["uuid", "version", "service"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_compatibility" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params["uuid"]:
            _path_params["uuid"] = _params["uuid"]
        if _params["version"]:
            _path_params["version"] = _params["version"]
        if _params["service"]:
            _path_params["service"] = _params["service"]

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # authentication setting
        _auth_settings = ["ApiKey"]

        _response_types_map = {
            "200": "V1Compatibility",
            "204": "object",
            "403": "object",
            "404": "object",
        }

        return self.api_client.call_api(
            "/api/v1/compatibility/{uuid}/{version}/{service}",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def get_installation(
        self,
        auth: Annotated[Optional[bool], Field(description="auth.")] = None,
        **kwargs
    ) -> V1Installation:
        """Get installation versions

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_installation(auth, async_req=True)
        >>> result = thread.get()

        :param auth: auth.
        :type auth: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: V1Installation
        """
        kwargs["_return_http_data_only"] = True
        return self.get_installation_with_http_info(auth, **kwargs)

    @validate_arguments
    def get_installation_with_http_info(
        self,
        auth: Annotated[Optional[bool], Field(description="auth.")] = None,
        **kwargs
    ):
        """Get installation versions

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_installation_with_http_info(auth, async_req=True)
        >>> result = thread.get()

        :param auth: auth.
        :type auth: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(V1Installation, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = ["auth"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_installation" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get("auth") is not None:  # noqa: E501
            _query_params.append(("auth", _params["auth"]))

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = ["ApiKey"]  # noqa: E501

        _response_types_map = {
            "200": "V1Installation",
            "204": "object",
            "403": "object",
            "404": "object",
        }

        return self.api_client.call_api(
            "/api/v1/installation",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def get_log_handler(self, **kwargs) -> V1LogHandler:  # noqa: E501
        """Get log handler versions  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_log_handler(async_req=True)
        >>> result = thread.get()

        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: V1LogHandler
        """
        kwargs["_return_http_data_only"] = True
        return self.get_log_handler_with_http_info(**kwargs)  # noqa: E501

    @validate_arguments
    def get_log_handler_with_http_info(self, **kwargs):  # noqa: E501
        """Get log handler versions  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_log_handler_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(V1LogHandler, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = []
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_log_handler" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = ["ApiKey"]  # noqa: E501

        _response_types_map = {
            "200": "V1LogHandler",
            "204": "object",
            "403": "object",
            "404": "object",
        }

        return self.api_client.call_api(
            "/api/v1/log_handler",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )



def set_build_fixture(config, owner):
    config["build"] = {
        "params": {
            "context": {"value": "/path/context"},
        },
        "runPatch": {
            "init": [
                {
                    "git": {"revision": "branch2"},
                    "connection": "connection-repo",
                }
            ]
        },
        "hubRef": f"{owner}/kaniko",
    }
    return config



from polyaxon._docker.converter.converters.job import JobConverter
from polyaxon._docker.converter.converters.service import ServiceConverter
from polyaxon._flow import V1RunKind

CONVERTERS = {
    V1RunKind.JOB: JobConverter,
    V1RunKind.SERVICE: ServiceConverter,
}



from polyaxon._flow import V1HpDateRange, V1MatrixKind, V1RunKind


def get_fxt_backfill_with_inputs_outputs():
    return {
        "version": 1.1,
        "kind": "operation",
        "name": "run",
        "tags": ["key1", "value1"],
        "params": {"image": {"value": "test"}, "lr": {"value": 0.001}},
        "matrix": {
            "kind": V1MatrixKind.GRID,
            "params": {
                "dts": V1HpDateRange(value=["2012-01-01", "2012-01-20", 5]).to_dict(),
            },
        },
        "component": {
            "name": "experiment-template",
            "description": "experiment to predict something",
            "tags": ["key", "value"],
            "inputs": [
                {"name": "lr", "type": "float", "value": 0.1, "isOptional": True},
                {"name": "image", "type": "str"},
                {"name": "dts", "type": "date"},
            ],
            "termination": {"maxRetries": 2},
            "run": {
                "kind": V1RunKind.JOB,
                "environment": {
                    "nodeSelector": {"polyaxon": "experiments"},
                    "serviceAccountName": "service",
                    "imagePullSecrets": ["secret1", "secret2"],
                },
                "container": {
                    "image": "{{ image }}",
                    "command": ["python3", "main.py"],
                    "args": ["--lr={{ lr }}", "{{ params.dts.as_arg }}"],
                    "resources": {"requests": {"cpu": 1}},
                },
            },
        },
    }



from typing import Optional

from clipped.compact.pydantic import Extra, Field, StrictStr

from polyaxon._env_vars.keys import ENV_KEYS_HOME
from polyaxon._schemas.base import BaseSchemaModel


class HomeConfig(BaseSchemaModel):
    """
    Home config for managing the main context path.


    Args:
        path: `str`. The context path where to write/read configs.
    """

    _IDENTIFIER = "home"

    path: Optional[StrictStr] = Field(alias=ENV_KEYS_HOME)

    class Config:
        extra = Extra.ignore



from typing import Optional, Union

from clipped.compact.pydantic import StrictStr
from clipped.types.numbers import StrictIntOrFloat
from clipped.utils.units import to_cpu_value, to_memory_bytes

from polyaxon._k8s.k8s_schemas import V1Container
from polyaxon._schemas.base import BaseSchemaModel


class V1RunResources(BaseSchemaModel):
    cpu: Optional[Union[StrictIntOrFloat, StrictStr]]
    memory: Optional[Union[StrictIntOrFloat, StrictStr]]
    gpu: Optional[Union[StrictIntOrFloat, StrictStr]]
    custom: Optional[Union[StrictIntOrFloat, StrictStr]]
    cost: Optional[Union[StrictIntOrFloat, StrictStr]]

    _MEMORY = "memory"
    _CPU = "cpu"
    _GPU = "gpu"
    _CUSTOM_RESOURCE = "custom"
    _COST = "cost"
    _VALUES = {_MEMORY, _CPU, _GPU, _CUSTOM_RESOURCE, _COST}

    @classmethod
    def validate_memory(cls, value):
        return to_memory_bytes(value or 0)

    @classmethod
    def validate_cpu(cls, value):
        return to_cpu_value(value or 0)

    @classmethod
    def validate_resource(cls, value):
        try:
            return float(value or 0)
        except (ValueError, TypeError):
            return 0

    @classmethod
    def from_container(cls, container: V1Container):
        result = cls(memory=0, cpu=0, gpu=0, custom=0, cost=0)
        if not container or not container.resources:
            return result

        resources = container.resources
        if hasattr(resources, "to_dict"):
            resources = resources.to_dict()

        requests = resources.get("requests") or {}
        limits = resources.get("limits") or {}

        for k, v in requests.items():
            if k == cls._MEMORY:
                result.memory = v
            elif k == cls._CPU:
                result.cpu = v
            elif cls._GPU in k:
                result.gpu = v
            else:
                result.custom = v
        # We only set limits if no requests is provided
        for k, v in limits.items():
            if k == cls._MEMORY:
                result.memory = result.memory if result.memory else v
            elif k == cls._CPU:
                result.cpu = result.cpu if result.cpu else v
            elif cls._GPU in k:
                result.gpu = result.gpu if result.gpu else v
            elif not result.custom:
                result.custom = v

        # Parse values
        result.memory = cls.validate_memory(result.memory)
        result.cpu = cls.validate_cpu(result.cpu)
        result.gpu = cls.validate_resource(result.gpu)
        result.custom = cls.validate_resource(result.custom)
        return result

    def __add__(self, other):
        if not other:
            return self

        def _get_value(v1, v2):
            return v1 or 0 + v2 or 0

        return V1RunResources(
            memory=_get_value(self.memory, other.memory),
            cpu=_get_value(self.cpu, other.cpu),
            gpu=_get_value(self.gpu, other.gpu),
            custom=_get_value(self.custom, other.custom),
            cost=_get_value(self.cost, other.cost),
        )



from typing import Dict, List, Optional, Tuple, Union

from clipped.compact.pydantic import Field
from clipped.utils.units import to_cpu_value, to_memory_bytes, to_unit_memory

from polyaxon._schemas.base import BaseSchemaModel


class V1EnvVar(BaseSchemaModel):
    __root__: Union[Tuple[str, str], Dict[str, str]]

    def to_cmd(self):
        if isinstance(self.__root__, tuple):
            value = self.__root__
        else:
            value = self.__root__.items()
        return [f"{value[0]}={value[1]}"]


class V1VolumeMount(BaseSchemaModel):
    __root__: Tuple[str, str]

    def to_cmd(self):
        return list(self.__root__)


class V1ContainerPort(BaseSchemaModel):
    __root__: Union[str, Tuple[str, str], Dict[str, str]]

    def to_cmd(self):
        if isinstance(self.__root__, str):
            return [self.__root__]

        if isinstance(self.__root__, tuple):
            value = self.__root__
        else:
            value = self.__root__.items()
        return list(value[0])


class V1ResourceRequirements(BaseSchemaModel):
    cpus: Optional[str]
    memory: Optional[str]
    gpus: Optional[str]

    @staticmethod
    def from_k8s_cpu(cpu: str) -> Union[str, float]:
        if not cpu:
            return cpu
        return to_cpu_value(cpu)

    @staticmethod
    def from_k8s_memory(memory: str) -> str:
        if not memory:
            return memory
        docker_mem_bytes = to_memory_bytes(memory)
        return to_unit_memory(number=docker_mem_bytes, use_i=False, use_space=False)

    def to_cmd(self):
        cmd_args = []
        if self.cpus:
            cmd_args += ["--cpus", self.cpus]
        if self.memory:
            cmd_args += ["--memory", self.memory]
        if self.gpus:
            cmd_args += ["--gpus", self.gpus]
        return cmd_args


class V1Container(BaseSchemaModel):
    image: Optional[str]
    name: Optional[str]
    command: Optional[List[str]]
    args: Optional[List[str]]
    env: Optional[List[V1EnvVar]]
    volume_mounts: Optional[List[V1VolumeMount]] = Field(alias="volumeMounts")
    resources: Optional[V1ResourceRequirements]
    ports: Optional[List[V1ContainerPort]]
    working_dir: Optional[str] = Field(alias="workingDir")

    def get_cmd_args(self):
        cmd_args = ["run", "--rm"]
        for env in self.env:
            cmd_args += ["-e"] + env.to_cmd()
        for volume in self.volume_mounts:
            cmd_args += volume.to_cmd()
        if self.working_dir:
            cmd_args += ["-w", self.working_dir]
        if self.resources:
            if self.resources.cpus:
                cmd_args += ["--cpus", self.resources.cpus]
            if self.resources.memory:
                cmd_args += ["--memory", self.resources.memory]
            if self.resources.gpus:
                cmd_args += ["--gpus", self.resources.gpus]
        if self.ports:
            for port in self.ports:
                cmd_args += ["-p"] + port.to_cmd()
        if self.command:
            cmd_args += ["--entrypoint", self.command[0]]
        cmd_args += [self.image]
        if self.command:
            cmd_args += self.command[1:]
        if self.args:
            cmd_args += self.args
        return cmd_args



from polyaxon._fs.fs import (
    get_artifacts_connection,
    get_fs_from_name,
    get_sync_fs_from_connection,
)
from polyaxon._fs.utils import get_store_path

# Backward compatibility
get_artifacts_connection_type = get_artifacts_connection



from typing import Any, Dict, Optional

from clipped.config.schema import BaseAllowSchemaModel

from polyaxon._schemas.lifecycle import V1Statuses
from polyaxon._sdk.schemas.v1_agent_state_response_agent_state import (
    V1AgentStateResponseAgentState,
)


class V1AgentStateResponse(BaseAllowSchemaModel):
    status: Optional[V1Statuses]
    state: Optional[V1AgentStateResponseAgentState]
    live_state: Optional[int]
    compatible_updates: Optional[Dict[str, Any]]



from typing import TYPE_CHECKING, Union

from clipped.formatting import Printer
from clipped.utils.enums import get_enum_value

from polyaxon._contexts import paths as ctx_paths
from polyaxon._fs.watcher import FSWatcher
from polyaxon.logger import logger

if TYPE_CHECKING:
    from polyaxon._connections import V1ConnectionKind


def sync_file_watcher(path: str):
    try:
        fw = FSWatcher()
        fw.sync(path)
        fw.write(ctx_paths.CONTEXT_MOUNT_FILE_WATCHER)
    except Exception as e:  # File watcher should not prevent job from starting
        logger.warning(
            "File watcher failed syncing path: {}.\nError: {}".format(path, e)
        )


def download_artifact(
    connection_name: str,
    connection_kind: Union[str, "V1ConnectionKind"],
    path_from: str,
    path_to: str,
    is_file: bool,
    raise_errors: bool,
    sync_fw: bool,
    check_path: bool,
):
    from polyaxon._fs.fs import get_fs_from_name
    from polyaxon._fs.manager import download_file_or_dir

    fs = get_fs_from_name(connection_name=connection_name)
    try:
        download_file_or_dir(
            fs=fs,
            path_from=path_from,
            path_to=path_to,
            is_file=is_file,
            check_path=check_path,
        )
        if sync_fw:
            sync_file_watcher(path_to)
        Printer.success(
            "{} path is initialized, path: `{}`".format(
                get_enum_value(connection_kind), path_to
            )
        )
    except Exception as e:
        if raise_errors:
            raise e
        else:
            logger.debug(
                "Initialization failed, the error was ignored. Error details %s", e
            )
